#!/usr/bin/env python

import os
import botocore.session
import configparser

from . import util


class Configuration:

    def __init__(self, **kwargs):
        self.options = {}
        self.__boto_session = botocore.session.Session()

        # Set up some defaults. These can be overridden as fit.
        self.ask_role = False
        self.duration = self.max_duration
        self.idp_id = None
        self.password = None
        self.profile = "sts"
        self.region = "ap-southeast-2"
        self.role_arn = None
        self.sp_id = None
        self.u2f_disabled = False
        self.username = None

    # For the "~/.aws/config" file, we use the format "[profile testing]"
    # for the 'testing' profile. The credential file will just be "[testing]"
    # in that case. See https://docs.aws.amazon.com/cli/latest/userguide/cli-multiple-profiles.html
    # for more information.
    @staticmethod
    def config_profile(profile):
        if str(profile).lower() == 'default':
            return profile
        else:
            return 'profile {}'.format(str(profile))

    @property
    def max_duration(self):
        return 3600

    @property
    def credentials_file(self):
        return os.path.expanduser(self.__boto_session.get_config_variable('credentials_file'))

    @property
    def config_file(self):
        return os.path.expanduser(self.__boto_session.get_config_variable('config_file'))

    def ensure_config_files_exist(self):
        for file in [self.config_file, self.credentials_file]:
            directory = os.path.dirname(file)
            if not os.path.exists(directory):
                os.mkdir(directory, 0o700)
            if not os.path.exists(file):
                util.Util.touch(file)

    # Will raise exceptions if the configuration is invalid, otherwise returns
    # None. Use this at any point to validate the configuration is in a good
    # state.
    def raise_if_invalid(self):
        # ask_role
        assert (self.ask_role.__class__ is bool), "Expected ask_role to be a boolean. Got {}.".format(self.ask_role.__class__)

        # duration
        assert (self.duration.__class__ is int), "Expected duration to be an integer. Got {}.".format(self.duration.__class__)
        assert (self.duration > 0), "Expected duration to be greater than 0. Got {}.".format(self.duration)
        assert (self.duration <= self.max_duration), "Expected duration to be less than or equal to max_duration ({}). Got {}.".format(self.max_duration, self.duration)

        # profile
        assert (self.profile.__class__ is str), "Expected profile to be a string. Got {}.".format(self.profile.__class__)

        # region
        assert (self.region.__class__ is str), "Expected region to be a string. Got {}.".format(self.region.__class__)

        # idp_id
        assert (self.idp_id is not None), "Expected idp_id to be set to non-None value."

        # sp_id
        assert (self.sp_id is not None), "Expected sp_id to be set to non-None value."

        # username
        assert (self.username.__class__ is str), "Expected username to be a string. Got {}.".format(self.username.__class__)

        # password
        assert (self.password.__class__ is str), "Expected password to be a string. Got {}.".format(self.password.__class__)

        # role_arn (Can be blank, we'll just prompt)
        if self.role_arn is not None:
            assert (self.role_arn.__class__ is str), "Expected role_arn to be None or a string. Got {}.".format(self.role_arn.__class__)
            assert ("arn:aws:iam::" in self.role_arn), "Expected role_arn to contain 'arn:aws:iam::'. Got '{}'.".format(self.role_arn)

        # u2f_disabled
        assert (self.u2f_disabled.__class__ is bool), "Expected u2f_disabled to be a boolean. Got {}.".format(self.u2f_disabled.__class__)

    # Write the configuration (and credentials) out to disk. This allows for
    # regular AWS tooling (aws cli and boto) to use the credentials in the
    # profile the user specified.
    def write(self, amazon_object):
        self.ensure_config_files_exist()

        assert (self.profile is not None), "Can not store config/credentials if the AWS_PROFILE is None."

        # Write to the configuration file
        profile = Configuration.config_profile(self.profile)
        config_parser = configparser.RawConfigParser()
        config_parser.read(self.config_file)
        if not config_parser.has_section(profile):
            config_parser.add_section(profile)
        config_parser.set(profile, 'region', self.region)
        config_parser.set(profile, 'google_config.ask_role', self.ask_role)
        config_parser.set(profile, 'google_config.duration', self.duration)
        config_parser.set(profile, 'google_config.google_idp_id', self.idp_id)
        config_parser.set(profile, 'google_config.role_arn', self.role_arn)
        config_parser.set(profile, 'google_config.google_sp_id', self.sp_id)
        config_parser.set(profile, 'google_config.u2f_disabled', self.u2f_disabled)
        config_parser.set(profile, 'google_config.google_username', self.username)
        with open(self.config_file, 'w+') as f:
            config_parser.write(f)

        # Write to the credentials file (only if we have credentials)
        if amazon_object is not None:
            credentials_parser = configparser.RawConfigParser()
            credentials_parser.read(self.credentials_file)
            if not credentials_parser.has_section(self.profile):
                credentials_parser.add_section(self.profile)
            credentials_parser.set(self.profile, 'aws_access_key_id', amazon_object.access_key_id)
            credentials_parser.set(self.profile, 'aws_secret_access_key', amazon_object.secret_access_key)
            credentials_parser.set(self.profile, 'aws_security_token', amazon_object.session_token)
            credentials_parser.set(self.profile, 'aws_session_expiration', amazon_object.expiration.strftime('%Y-%m-%dT%H:%M:%S%z'))
            credentials_parser.set(self.profile, 'aws_session_token', amazon_object.session_token)
            with open(self.credentials_file, 'w+') as f:
                credentials_parser.write(f)

    # Read from the configuration file and override ALL values currently stored
    # in the configuration object. As this is potentially destructive, it's
    # important to only run this in the beginning of the object initialization.
    # We do not read AWS credentials, as this tool's use case is to obtain
    # them.
    def read(self, profile):
        self.ensure_config_files_exist()

        # Shortening Convenience functions
        coalesce = util.Util.coalesce
        unicode_to_string = util.Util.unicode_to_string_if_needed

        profile_string = Configuration.config_profile(profile)
        config_parser = configparser.RawConfigParser()
        config_parser.read(self.config_file)

        if config_parser.has_section(profile_string):
            self.profile = profile

            # Ask Role
            read_ask_role = config_parser[profile_string].getboolean('google_config.ask_role', None)
            self.ask_role = coalesce(read_ask_role, self.ask_role)

            # Duration
            read_duration = config_parser[profile_string].getint('google_config.duration', None)
            self.duration = coalesce(read_duration, self.duration)

            # IDP ID
            read_idp_id = unicode_to_string(config_parser[profile_string].get('google_config.google_idp_id', None))
            self.idp_id = coalesce(read_idp_id, self.idp_id)

            # Region
            read_region = unicode_to_string(config_parser[profile_string].get('region', None))
            self.region = coalesce(read_region, self.region)

            # Role ARN
            read_role_arn = unicode_to_string(config_parser[profile_string].get('google_config.role_arn', None))
            self.role_arn = coalesce(read_role_arn, self.role_arn)

            # SP ID
            read_sp_id = unicode_to_string(config_parser[profile_string].get('google_config.google_sp_id', None))
            self.sp_id = coalesce(read_sp_id, self.sp_id)

            # U2F Disabled
            read_u2f_disabled = config_parser[profile_string].getboolean('google_config.u2f_disabled', None)
            self.u2f_disabled = coalesce(read_u2f_disabled, self.u2f_disabled)

            # Username
            read_username = unicode_to_string(config_parser[profile_string].get('google_config.google_username', None))
            self.username = coalesce(read_username, self.username)



#!/usr/bin/env python

import base64
import unittest

from aws_google_auth import amazon
from aws_google_auth import configuration
from os import path


class TestAmazon(unittest.TestCase):

    def valid_config(self):
        return configuration.Configuration(
            idp_id="IDPID",
            sp_id="SPID",
            username="user@example.com",
            password="hunter2")

    def valid_saml_response(self):
        here = path.abspath(path.dirname(__file__))
        with open(path.join(here, 'valid-response.xml')) as fp:
            return base64.b64encode(fp.read().encode('utf-8'))
        return None

    def extra_comma_saml_response(self):
        here = path.abspath(path.dirname(__file__))
        with open(path.join(here, 'too-many-commas.xml')) as fp:
            return base64.b64encode(fp.read().encode('utf-8'))
        return None

    def test_sts_client(self):
        a = amazon.Amazon(self.valid_config(), "encoded-saml")
        self.assertEqual(str(a.sts_client.__class__), "<class 'botocore.client.STS'>")

    def test_role_extraction(self):
        a = amazon.Amazon(self.valid_config(), self.valid_saml_response())
        self.assertIsInstance(a.roles, dict)
        list_of_testing_roles = [
            "arn:aws:iam::123456789012:role/admin",
            "arn:aws:iam::123456789012:role/read-only",
            "arn:aws:iam::123456789012:role/test"]
        self.assertEqual(sorted(list(a.roles.keys())), sorted(list_of_testing_roles))

    def test_role_extraction_too_many_commas(self):
        # See https://github.com/cevoaustralia/aws-google-auth/issues/12
        a = amazon.Amazon(self.valid_config(), self.extra_comma_saml_response())
        self.assertIsInstance(a.roles, dict)
        list_of_testing_roles = [
            "arn:aws:iam::123456789012:role/admin",
            "arn:aws:iam::123456789012:role/read-only",
            "arn:aws:iam::123456789012:role/test"]
        self.assertEqual(sorted(list(a.roles.keys())), sorted(list_of_testing_roles))



#!/usr/bin/env python

import os


class Util:

    @staticmethod
    def get_input(prompt):
        try:
            return raw_input(prompt)
        except NameError:
            return input(prompt)

    @staticmethod
    def pick_a_role(roles):
        while True:
            for i, role in enumerate(roles):
                print("[{:>3d}] {}".format(i + 1, role))

            prompt = 'Type the number (1 - {:d}) of the role to assume: '.format(len(roles))
            choice = Util.get_input(prompt)

            try:
                return list(roles.items())[int(choice) - 1]
            except IndexError:
                print("Invalid choice, try again.")

    @staticmethod
    def touch(file_name, mode=0o600):
        flags = os.O_CREAT | os.O_APPEND
        with os.fdopen(os.open(file_name, flags, mode)) as f:
            try:
                os.utime(file_name, None)
            finally:
                f.close()

    # This method returns the first non-None value in args. If all values are
    # None, None will be returned. If there are no arguments, None will be
    # returned.
    @staticmethod
    def coalesce(*args):
        for number, value in enumerate(args):
            if value is not None:
                return value
        return None

    @staticmethod
    def unicode_to_string_if_needed(object):
        if "unicode" in str(object.__class__):
            return object.encode('utf-8')
        else:
            return object



#!/usr/bin/env python

import boto3
import base64
from lxml import etree


class Amazon:

    def __init__(self, config, encoded_saml):
        self.config = config
        self.encoded_saml = encoded_saml
        self.__token = None

    @property
    def sts_client(self):
        return boto3.client('sts', region_name=self.config.region)

    @property
    def decoded_saml(self):
        return base64.b64decode(self.encoded_saml)

    @property
    def token(self):
        if self.__token is None:
            self.__token = self.sts_client.assume_role_with_saml(
                RoleArn=self.config.role_arn,
                PrincipalArn=self.config.provider,
                SAMLAssertion=self.encoded_saml,
                DurationSeconds=self.config.duration)
        return self.__token

    @property
    def access_key_id(self):
        return self.token['Credentials']['AccessKeyId']

    @property
    def secret_access_key(self):
        return self.token['Credentials']['SecretAccessKey']

    @property
    def session_token(self):
        return self.token['Credentials']['SessionToken']

    @property
    def expiration(self):
        return self.token['Credentials']['Expiration']

    def print_export_line(self):
        export_template = "export AWS_ACCESS_KEY_ID='{}' AWS_SECRET_ACCESS_KEY='{}' AWS_SESSION_TOKEN='{}' AWS_SESSION_EXPIRATION='{}'"

        formatted = export_template.format(
            self.access_key_id,
            self.secret_access_key,
            self.session_token,
            self.expiration.strftime('%Y-%m-%dT%H:%M:%S%z'))

        print(formatted)

    @property
    def roles(self):
        doc = etree.fromstring(self.decoded_saml)
        roles = {}
        for x in doc.xpath('//*[@Name = "https://aws.amazon.com/SAML/Attributes/Role"]//text()'):
            if "arn:aws:iam:" in x:
                res = x.split(',')
                roles[res[0]] = res[1]
        return roles



#!/usr/bin/env python

from . import _version
from . import configuration
from . import util
from . import google
from . import amazon

import argparse
import getpass
import os
import sys
from tzlocal import get_localzone


def parse_args(args):
    parser = argparse.ArgumentParser(
        prog="aws-google-auth",
        description="Acquire temporary AWS credentials via Google SSO",
    )

    parser.add_argument('-u', '--username', help='Google Apps username ($GOOGLE_USERNAME)')
    parser.add_argument('-I', '--idp-id', help='Google SSO IDP identifier ($GOOGLE_IDP_ID)')
    parser.add_argument('-S', '--sp-id', help='Google SSO SP identifier ($GOOGLE_SP_ID)')
    parser.add_argument('-R', '--region', help='AWS region endpoint ($AWS_DEFAULT_REGION)')
    parser.add_argument('-d', '--duration', type=int, help='Credential duration ($DURATION)')
    parser.add_argument('-p', '--profile', help='AWS profile (defaults to value of $AWS_PROFILE, then falls back to \'sts\')')
    parser.add_argument('-D', '--disable-u2f', action='store_true', help='Disable U2F functionality.')

    role_group = parser.add_mutually_exclusive_group()
    role_group.add_argument('-a', '--ask-role', action='store_true', help='Set true to always pick the role')
    role_group.add_argument('-r', '--role-arn', help='The ARN of the role to assume')
    parser.add_argument('-V', '--version', action='version',
                        version='%(prog)s {version}'.format(version=_version.__version__))

    return parser.parse_args(args)


def exit_if_unsupported_python():
    if sys.version_info[0] == 2 and sys.version_info[1] < 7:
        print("aws-google-auth requires Python 2.7 or higher. Please consider upgrading. Support "
              "for Python 2.6 and lower was dropped because this tool's dependencies dropped support.")
        print("")
        print("For debugging, it appears you're running: '{}'.".format(str(sys.version_info)))
        print("")
        print("See https://github.com/cevoaustralia/aws-google-auth/issues/41 for more information.")
        sys.exit(1)


def main():
    try:
        cli(sys.argv[1:])
    except KeyboardInterrupt:
        pass


def cli(cli_args):
    exit_if_unsupported_python()

    # Shortening Convenience functions
    coalesce = util.Util.coalesce

    args = parse_args(args=cli_args)

    # Create a blank configuration object (has the defaults pre-filled)
    config = configuration.Configuration()

    # Have the configuration update itself via the ~/.aws/config on disk.
    # Profile (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.profile = coalesce(
        args.profile,
        os.getenv('AWS_PROFILE'),
        config.profile)

    # Now that we've established the profile, we can read the configuration and
    # fill in all the other variables.
    config.read(config.profile)

    # Ask Role (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.ask_role = coalesce(
        args.ask_role,
        os.getenv('AWS_ASK_ROLE'),
        config.ask_role)

    # Duration (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.duration = coalesce(
        args.duration,
        os.getenv('DURATION'),
        config.duration)

    # IDP ID (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.idp_id = coalesce(
        args.idp_id,
        os.getenv('GOOGLE_IDP_ID'),
        config.idp_id)

    # Region (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.region = coalesce(
        args.region,
        os.getenv('AWS_DEFAULT_REGION'),
        config.region)

    # ROLE ARN (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.role_arn = coalesce(
        args.role_arn,
        os.getenv('AWS_ROLE_ARN'),
        config.role_arn)

    # SP ID (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.sp_id = coalesce(
        args.sp_id,
        os.getenv('GOOGLE_SP_ID'),
        config.sp_id)

    # U2F Disabled (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.u2f_disabled = coalesce(
        args.disable_u2f,
        os.getenv('U2F_DISABLED'),
        config.u2f_disabled)

    # Username (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.username = coalesce(
        args.username,
        os.getenv('GOOGLE_USERNAME'),
        config.username)

    # There are some mandatory arguments. Make sure the user supplied them.
    if config.username is None:
        config.username = util.Util.get_input("Google username: ")
    if config.idp_id is None:
        config.idp_id = util.Util.get_input("Google IDP ID: ")
    if config.sp_id is None:
        config.sp_id = util.Util.get_input("Google SP ID: ")

    # There is no way (intentional) to pass in the password via the command
    # line nor environment variables. This prevents password leakage.
    config.password = getpass.getpass("Google Password: ")

    # Validate Options
    try:
        config.raise_if_invalid()
    except AssertionError:
        print("Invalid parameters.")
        raise

    google_client = google.Google(config)
    google_client.do_login()
    encoded_saml = google_client.parse_saml()

    amazon_client = amazon.Amazon(config, encoded_saml)
    roles = amazon_client.roles

    # Determine the provider and the role arn (if the the user provided isn't an option)
    if config.role_arn in roles and not config.ask_role:
        config.provider = roles[config.role_arn]
    else:
        config.role_arn, config.provider = util.Util.pick_a_role(roles)

    print("Assuming " + config.role_arn)
    print("Credentials Expiration: " + format(amazon_client.expiration.astimezone(get_localzone())))

    amazon_client.print_export_line()
    config.write(amazon_client)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from . import _version

import sys
import requests
import json
from bs4 import BeautifulSoup

# In Python3, the library 'urlparse' was renamed to 'urllib.parse'. For this to
# maintain compatibility with both Python 2 and Python 3, the import must be
# dynamically chosen based on the version detected.
if sys.version_info >= (3, 0):
    import urllib.parse as urlparse
else:
    import urlparse

# The U2F USB Library is optional, if it's there, include it.
try:
    from . import u2f
except ImportError:
    print("Failed to import U2F libraries, U2F login unavailable. Other "
          "methods can still continue.")


class Google:
    def __init__(self, config):
        """The Google object holds authentication state
        for a given session. You need to supply:

        username: FQDN Google username, eg first.last@example.com
        password: obvious
        idp_id: Google's assigned IdP identifier for your G-suite account
        sp_id: Google's assigned SP identifier for your AWS SAML app

        Optionally, you can supply:
        duration_seconds: number of seconds for the session to be active (max 3600)
        """

        self.version = _version.__version__
        self.config = config

    @property
    def login_url(self):
        return "https://accounts.google.com/o/saml2/initsso?idpid={}&spid={}&forceauthn=false".format(self.config.idp_id, self.config.sp_id)

    def do_login(self):
        self.session = requests.Session()
        self.session.headers['User-Agent'] = "AWS Sign-in/{} (Cevo aws-google-auth)".format(self.version)
        sess = self.session.get(self.login_url)
        sess.raise_for_status()

        # Collect information from the page source
        first_page = BeautifulSoup(sess.text, 'html.parser')
        gxf = first_page.find('input', {'name': 'gxf'}).get('value')
        self.cont = first_page.find('input', {'name': 'continue'}).get('value')
        page = first_page.find('input', {'name': 'Page'}).get('value')
        sign_in = first_page.find('input', {'name': 'signIn'}).get('value')
        account_login_url = first_page.find('form', {'id': 'gaia_loginform'}).get('action')

        payload = {
            'bgresponse': 'js_disabled',
            'checkConnection': '',
            'checkedDomains': 'youtube',
            'continue': self.cont,
            'Email': self.config.username,
            'gxf': gxf,
            'identifier-captcha-input': '',
            'identifiertoken': '',
            'identifiertoken_audio': '',
            'ltmpl': 'popup',
            'oauth': 1,
            'Page': page,
            'Passwd': '',
            'PersistentCookie': 'yes',
            'ProfileInformation': '',
            'pstMsg': 0,
            'sarp': 1,
            'scc': 1,
            'SessionState': '',
            'signIn': sign_in,
            '_utf8': '?',
        }

        # GALX is sometimes not there
        try:
            galx = first_page.find('input', {'name': 'GALX'}).get('value')
            payload['GALX'] = galx
        except:
            pass

        # POST to account login info page, to collect profile and session info
        sess = self.session.post(account_login_url, data=payload)
        sess.raise_for_status()
        self.session.headers['Referer'] = sess.url

        # Collect ProfileInformation, SessionState, signIn, and Password Challenge URL
        challenge_page = BeautifulSoup(sess.text, 'html.parser')

        profile_information = challenge_page.find('input', {'name': 'ProfileInformation'}).get('value')
        session_state = challenge_page.find('input', {'name': 'SessionState'}).get('value')
        sign_in = challenge_page.find('input', {'name': 'signIn'}).get('value')
        passwd_challenge_url = challenge_page.find('form', {'id': 'gaia_loginform'}).get('action')

        # Update the payload
        payload['SessionState'] = session_state
        payload['ProfileInformation'] = profile_information
        payload['signIn'] = sign_in
        payload['Passwd'] = self.config.password

        # POST to Authenticate Password
        sess = self.session.post(passwd_challenge_url, data=payload)
        sess.raise_for_status()
        response_page = BeautifulSoup(sess.text, 'html.parser')
        error = response_page.find(class_='error-msg')
        cap = response_page.find('input', {'name': 'logincaptcha'})

        # Were there any errors logging in? Could be invalid username or password
        # There could also sometimes be a Captcha, which means Google thinks you,
        # or someone using the same outbound IP address as you, is a bot.
        if error is not None:
            raise ValueError('Invalid username or password')

        if cap is not None:
            raise ValueError('Captcha Required. Manually Login to remove this.')

        self.session.headers['Referer'] = sess.url

        # Was there an MFA challenge?
        if "challenge/totp/" in sess.url:
            sess = self.handle_totp(sess)
        elif "challenge/ipp/" in sess.url:
            sess = self.handle_sms(sess)
        elif "challenge/az/" in sess.url:
            sess = self.handle_prompt(sess)
        elif "challenge/sk/" in sess.url:
            sess = self.handle_sk(sess)

        # ... there are different URLs for backup codes (printed)
        # and security keys (eg yubikey) as well
        # save for later
        self.session_state = sess

    def parse_saml(self):
        if self.session_state is None:
            raise RuntimeError('You must use do_login() before calling parse_saml()')

        parsed = BeautifulSoup(self.session_state.text, 'html.parser')
        try:
            saml_element = parsed.find('input', {'name': 'SAMLResponse'}).get('value')
        except:
            raise RuntimeError('Could not find SAML response, check your credentials')

        return saml_element

    def handle_sk(self, sess):
        response_page = BeautifulSoup(sess.text, 'html.parser')
        challenge_url = sess.url.split("?")[0]

        challenges_txt = response_page.find('input', {'name': "id-challenge"}).get('value')
        challenges = json.loads(challenges_txt)

        facet_url = urlparse.urlparse(challenge_url)
        facet = facet_url.scheme + "://" + facet_url.netloc
        app_id = challenges["appId"]
        u2f_challenges = []
        for c in challenges["challenges"]:
            c["appId"] = app_id
            u2f_challenges.append(c)

        # Prompt the user up to attempts_remaining times to insert their U2F device.
        attempts_remaining = 5
        auth_response = None
        while True:
            try:
                auth_response = json.dumps(u2f.u2f_auth(u2f_challenges, facet))
                break
            except RuntimeWarning:
                print("No U2F device found. {} attempts remaining.".format(attempts_remaining))
                if attempts_remaining <= 0:
                    break
                else:
                    input("Insert your U2F device and press enter to try again...")
                    attempts_remaining -= 1

        # If we exceed the number of attempts, raise an error and let the program exit.
        if auth_response is None:
            raise RuntimeError("No U2F device found. Please check your setup.")

        payload = {
            'challengeId': response_page.find('input', {'name': 'challengeId'}).get('value'),
            'challengeType': response_page.find('input', {'name': 'challengeType'}).get('value'),
            'continue': response_page.find('input', {'name': 'continue'}).get('value'),
            'scc': response_page.find('input', {'name': 'scc'}).get('value'),
            'sarp': response_page.find('input', {'name': 'sarp'}).get('value'),
            'checkedDomains': response_page.find('input', {'name': 'checkedDomains'}).get('value'),
            'pstMsg': response_page.find('input', {'name': 'pstMsg'}).get('value'),
            'TL': response_page.find('input', {'name': 'TL'}).get('value'),
            'gxf': response_page.find('input', {'name': 'gxf'}).get('value'),
            'id-challenge': challenges_txt,
            'id-assertion': auth_response,
            'TrustDevice': 'on',
        }

        sess = self.session.post(challenge_url, data=payload)
        sess.raise_for_status()

        return sess

    def handle_sms(self, sess):
        response_page = BeautifulSoup(sess.text, 'html.parser')
        challenge_url = sess.url.split("?")[0]

        try:
            sms_token = raw_input("Enter SMS token: G-") or None
        except NameError:
            sms_token = input("Enter SMS token: G-") or None

        payload = {
            'challengeId': response_page.find('input', {'name': 'challengeId'}).get('value'),
            'challengeType': response_page.find('input', {'name': 'challengeType'}).get('value'),
            'continue': response_page.find('input', {'name': 'continue'}).get('value'),
            'scc': response_page.find('input', {'name': 'scc'}).get('value'),
            'sarp': response_page.find('input', {'name': 'sarp'}).get('value'),
            'checkedDomains': response_page.find('input', {'name': 'checkedDomains'}).get('value'),
            'pstMsg': response_page.find('input', {'name': 'pstMsg'}).get('value'),
            'TL': response_page.find('input', {'name': 'TL'}).get('value'),
            'gxf': response_page.find('input', {'name': 'gxf'}).get('value'),
            'Pin': sms_token,
            'TrustDevice': 'on',
        }

        # Submit IPP (SMS code)
        sess = self.session.post(challenge_url, data=payload)
        sess.raise_for_status()

        return sess

    def handle_prompt(self, sess):
        response_page = BeautifulSoup(sess.text, 'html.parser')
        challenge_url = sess.url.split("?")[0]

        data_key = response_page.find('div', {'data-api-key': True}).get('data-api-key')
        data_tx_id = response_page.find('div', {'data-tx-id': True}).get('data-tx-id')

        # Need to post this to the verification/pause endpoint
        await_url = "https://content.googleapis.com/cryptauth/v1/authzen/awaittx?alt=json&key=%s" % data_key
        await_body = {'txId': data_tx_id}

        print("Open the Google App, and tap 'Yes' on the prompt to sign in ...")

        self.session.headers['Referer'] = sess.url
        response = self.session.post(await_url, json=await_body)
        parsed = json.loads(response.text)

        payload = {
            'challengeId': response_page.find('input', {'name': 'challengeId'}).get('value'),
            'challengeType': response_page.find('input', {'name': 'challengeType'}).get('value'),
            'continue': response_page.find('input', {'name': 'continue'}).get('value'),
            'scc': response_page.find('input', {'name': 'scc'}).get('value'),
            'sarp': response_page.find('input', {'name': 'sarp'}).get('value'),
            'checkedDomains': response_page.find('input', {'name': 'checkedDomains'}).get('value'),
            'checkConnection': 'youtube:1295:1',
            'pstMsg': response_page.find('input', {'name': 'pstMsg'}).get('value'),
            'TL': response_page.find('input', {'name': 'TL'}).get('value'),
            'gxf': response_page.find('input', {'name': 'gxf'}).get('value'),
            'token': parsed['txToken'],
            'action': response_page.find('input', {'name': 'action'}).get('value'),
            'TrustDevice': 'on',
        }

        sess = self.session.post(challenge_url, data=payload)
        sess.raise_for_status()

        return sess

    def handle_totp(self, sess):
        response_page = BeautifulSoup(sess.text, 'html.parser')
        tl = response_page.find('input', {'name': 'TL'}).get('value')
        gxf = response_page.find('input', {'name': 'gxf'}).get('value')
        challenge_url = sess.url.split("?")[0]
        challenge_id = challenge_url.split("totp/")[1]

        try:
            mfa_token = raw_input("MFA token: ") or None
        except NameError:
            mfa_token = input("MFA token: ") or None

        if not mfa_token:
            raise ValueError("MFA token required for {} but none supplied.".format(self.config.username))

        payload = {
            'challengeId': challenge_id,
            'challengeType': 6,
            'continue': self.cont,
            'scc': 1,
            'sarp': 1,
            'checkedDomains': 'youtube',
            'pstMsg': 0,
            'TL': tl,
            'gxf': gxf,
            'Pin': mfa_token,
            'TrustDevice': 'on',
        }

        # Submit TOTP
        sess = self.session.post(challenge_url, data=payload)
        sess.raise_for_status()

        return sess



#!/usr/bin/env python

import unittest
from aws_google_auth import configuration


class TestConfigurationMethods(unittest.TestCase):

    def test_config_profile(self):
        self.assertEqual(configuration.Configuration.config_profile('default'), 'default')
        self.assertEqual(configuration.Configuration.config_profile('DEFAULT'), 'DEFAULT')
        self.assertEqual(configuration.Configuration.config_profile('testing'), 'profile testing')
        self.assertEqual(configuration.Configuration.config_profile(None), 'profile None')
        self.assertEqual(configuration.Configuration.config_profile(123456), 'profile 123456')

    def test_duration_invalid_values(self):
        # Duration must be an integer
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.password = "hunter2"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.duration = "bad_type"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected duration to be an integer.", str(e.exception))

        # Duration can not be negative
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.duration = -1
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected duration to be greater than 0.", str(e.exception))

        # Duration can not be greater than MAX_DURATION
        valid = configuration.Configuration()
        valid.idp_id = "sample_idp_id"
        c.password = "hunter2"
        valid.sp_id = "sample_sp_id"
        valid.username = "sample_username"
        valid.duration = 100
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.duration = (valid.max_duration + 1)
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected duration to be less than or equal to max_duration", str(e.exception))

    def test_duration_valid_values(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.duration = 100
        self.assertEqual(c.duration, 100)
        c.raise_if_invalid()
        c.duration = c.max_duration
        self.assertEqual(c.duration, c.max_duration)
        c.raise_if_invalid()
        c.duration = (c.max_duration - 1)
        self.assertEqual(c.duration, c.max_duration - 1)
        c.raise_if_invalid()

    def test_duration_defaults_to_max_duration(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        self.assertEqual(c.duration, c.max_duration)
        c.raise_if_invalid()

    def test_ask_role_invalid_values(self):
        # ask_role must be a boolean
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.ask_role = "bad_value"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected ask_role to be a boolean.", str(e.exception))

    def test_ask_role_valid_values(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.ask_role = True
        self.assertTrue(c.ask_role)
        c.raise_if_invalid()
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.password = "hunter2"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.ask_role = False
        self.assertFalse(c.ask_role)
        c.raise_if_invalid()

    def test_ask_role_optional(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        self.assertFalse(c.ask_role)
        c.raise_if_invalid()

    def test_idp_id_invalid_values(self):
        # idp_id must not be None
        c = configuration.Configuration()
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected idp_id to be set to non-None value.", str(e.exception))

    def test_idp_id_valid_values(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        self.assertEqual(c.idp_id, "sample_idp_id")
        c.raise_if_invalid()
        c.idp_id = 123456
        self.assertEqual(c.idp_id, 123456)
        c.raise_if_invalid()

    def test_sp_id_invalid_values(self):
        # sp_id must not be None
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected sp_id to be set to non-None value.", str(e.exception))

    def test_username_valid_values(self):
        c = configuration.Configuration()
        c.password = "hunter2"
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        self.assertEqual(c.username, "sample_username")
        c.raise_if_invalid()
        c.username = "123456"
        self.assertEqual(c.username, "123456")
        c.raise_if_invalid()

    def test_username_invalid_values(self):
        # username must be set
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.password = "hunter2"
        c.sp_id = "sample_sp_id"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected username to be a string.", str(e.exception))
        # username must be be string
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = 123456
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected username to be a string.", str(e.exception))

    def test_password_valid_values(self):
        c = configuration.Configuration()
        c.password = "hunter2"
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        self.assertEqual(c.password, "hunter2")
        c.raise_if_invalid()
        c.password = "123456"
        self.assertEqual(c.password, "123456")
        c.raise_if_invalid()

    def test_password_invalid_values(self):
        # password must be set
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.username = "sample_username"
        c.sp_id = "sample_sp_id"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected password to be a string.", str(e.exception))
        # password must be be string
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = 123456
        c.username = "sample_username"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected password to be a string.", str(e.exception))

    def test_sp_id_valid_values(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.password = "hunter2"
        self.assertEqual(c.sp_id, "sample_sp_id")
        c.raise_if_invalid()
        c.sp_id = 123456
        self.assertEqual(c.sp_id, 123456)
        c.raise_if_invalid()

    def test_profile_defaults_to_sts(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.password = "hunter2"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        self.assertEqual(c.profile, "sts")
        c.raise_if_invalid()

    def test_profile_invalid_values(self):
        # profile must be a string
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.profile = 123456
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected profile to be a string.", str(e.exception))

    def test_profile_valid_values(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.password = "hunter2"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.profile = "default"
        self.assertEqual(c.profile, "default")
        c.raise_if_invalid()
        c.profile = "sts"
        self.assertEqual(c.profile, "sts")
        c.raise_if_invalid()

    def test_profile_defaults(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.password = "hunter2"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        self.assertEqual(c.profile, 'sts')
        c.raise_if_invalid()

    def test_region_invalid_values(self):
        # region must be a string
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.region = 1234
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected region to be a string.", str(e.exception))

    def test_region_valid_values(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.region = "us-east-1"
        self.assertEqual(c.region, "us-east-1")
        c.raise_if_invalid()
        c.region = "us-west-2"
        self.assertEqual(c.region, "us-west-2")
        c.raise_if_invalid()

    def test_region_defaults_to_ap_southeast_2(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.password = "hunter2"
        self.assertEqual(c.region, "ap-southeast-2")
        c.raise_if_invalid()

    def test_role_arn_invalid_values(self):
        # role_arn must be a string
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.role_arn = 1234
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected role_arn to be None or a string.", str(e.exception))

        # role_arn be a arn-looking string
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        c.role_arn = "bad_string"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected role_arn to contain 'arn:aws:iam::'", str(e.exception))

    def test_role_arn_is_optional(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.password = "hunter2"
        c.username = "sample_username"
        self.assertIsNone(c.role_arn)
        c.raise_if_invalid()

    def test_role_arn_valid_values(self):
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.password = "hunter2"
        c.role_arn = "arn:aws:iam::some_arn_1"
        self.assertEqual(c.role_arn, "arn:aws:iam::some_arn_1")
        c.raise_if_invalid()
        c.role_arn = "arn:aws:iam::some_other_arn_2"
        self.assertEqual(c.role_arn, "arn:aws:iam::some_other_arn_2")
        c.raise_if_invalid()

    def test_u2f_disabled_invalid_values(self):
        # u2f_disabled must be a boolean
        c = configuration.Configuration()
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.password = "hunter2"
        c.u2f_disabled = 1234
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected u2f_disabled to be a boolean.", str(e.exception))

    def test_u2f_disabled_valid_values(self):
        c = configuration.Configuration()
        c.password = "hunter2"
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.u2f_disabled = True
        self.assertTrue(c.u2f_disabled)
        c.raise_if_invalid()
        c = configuration.Configuration()
        c.password = "hunter2"
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        c.u2f_disabled = False
        self.assertFalse(c.u2f_disabled)
        c.raise_if_invalid()

    def test_u2f_disabled_is_optional(self):
        c = configuration.Configuration()
        c.password = "hunter2"
        c.idp_id = "sample_idp_id"
        c.sp_id = "sample_sp_id"
        c.username = "sample_username"
        self.assertFalse(c.u2f_disabled)
        c.raise_if_invalid()
